#!/usr/bin/python3
import wreport
import unittest


class Var(unittest.TestCase):
    def setUp(self):
        self.table = wreport.Vartable.get_bufr(master_table_version_number=24)

    def testUndefCreation(self):
        var = wreport.Var(self.table["B01001"])
        self.assertEqual(var.code, "B01001")
        self.assertEqual(var.info.code, "B01001")
        self.assertFalse(var.isset)

    def testIntCreation(self):
        var = wreport.Var(self.table["B05001"], 12)
        self.assertEqual(var.code, "B05001")
        self.assertEqual(var.isset, True)
        self.assertEqual(var.enqi(), 12)
        self.assertEqual(var.enqd(), 0.00012)
        self.assertEqual(var.enqc(), "12")

    def testFloatCreation(self):
        var = wreport.Var(self.table["B05001"], 12.4)
        self.assertEqual(var.code, "B05001")
        self.assertEqual(var.isset, True)
        self.assertEqual(var.enqi(), 1240000)
        self.assertEqual(var.enqd(), 12.4)
        self.assertEqual(var.enqc(), "1240000")

    def testStringCreation(self):
        var = wreport.Var(self.table["B05001"], "123456")
        self.assertEqual(var.code, "B05001")
        self.assertEqual(var.isset, True)
        self.assertEqual(var.enqi(), 123456)
        self.assertEqual(var.enqd(), 1.23456)
        self.assertEqual(var.enqc(), "123456")

    def testStringification(self):
        var = wreport.Var(self.table["B01001"])
        self.assertEqual(str(var), "None")
        self.assertEqual(repr(var), "Var('B01001', None)")
        self.assertEqual(var.format(), "")
        self.assertEqual(var.format("foo"), "foo")

        var = wreport.Var(self.table["B05001"], 12.4)
        self.assertEqual(str(var), "12.40000")
        self.assertEqual(repr(var), "Var('B05001', 12.40000)")
        self.assertEqual(var.format("foo"), "12.40000")

    def testEnq(self):
        var = wreport.Var(self.table["B01001"], 1)
        self.assertEqual(type(var.enq()), int)
        self.assertEqual(var.enq(), 1)
        var = wreport.Var(self.table["B05001"], 1.12345)
        self.assertEqual(type(var.enq()), float)
        self.assertEqual(var.enq(), 1.12345)
        var = wreport.Var(self.table["B01019"], "ciao")
        self.assertIsInstance(var.enq(), str)
        self.assertEqual(var.enq(), "ciao")

    def testGet(self):
        var = wreport.Var(self.table["B01001"])
        self.assertIsNone(var.get())
        self.assertIs(var.get("foo"), "foo")
        var = wreport.Var(self.table["B01001"], 1)
        self.assertIs(var.get(), 1)
        var = wreport.Var(self.table["B05001"], 1.12345)
        self.assertEqual(var.get(), 1.12345)
        var = wreport.Var(self.table["B01019"], "ciao")
        self.assertEqual(var.get(), "ciao")

    def testEq(self):
        var = wreport.Var(self.table["B01001"], 1)
        self.assertEqual(var, var)
        self.assertEqual(var, wreport.Var(self.table["B01001"], 1))
        self.assertNotEqual(var, wreport.Var(self.table["B01001"], 2))
        self.assertNotEqual(var, wreport.Var(self.table["B01002"], 1))
        self.assertIsNot(var, None)
        self.assertIsNot(wreport.Var(self.table["B01001"]), None)

    def testAttrs(self):
        var = wreport.Var(self.table["B01001"], 1)

        # Querying a nonexisting attribute returns None
        self.assertIsNone(var.enqa("B33007"))

        # Querying attributes when there are none returns an empty list
        self.assertEqual(var.get_attrs(), [])

        # Removing a non existing attribute does nothing
        var.unseta("B33007")

        # Add an attribute
        var.seta(wreport.Var(self.table["B33007"], 50))

        # Query it
        attr = var.enqa("B33007")
        self.assertEqual(attr.code, "B33007")
        self.assertEqual(attr.enqd(), 50.0)

        # List attributes
        attrs = var.get_attrs()
        self.assertEqual(len(attrs), 1)
        self.assertEqual(attrs[0].code, "B33007")
        self.assertEqual(attrs[0].enqd(), 50.0)

        # Remove it
        var.unseta("B33007")
        self.assertIsNone(var.enqa("B33007"))
        self.assertEqual(var.get_attrs(), [])



#!/usr/bin/python3
import wreport
import unittest
import os


class Vartable(unittest.TestCase):
    def testEmpty(self):
        with self.assertRaises(NotImplementedError):
            wreport.Vartable()

    def testLoadBufr(self):
        t = wreport.Vartable.load_bufr(os.path.join(os.environ["WREPORT_TABLES"], "B0000000000000024000.txt"))
        self.assertEqual(t["B12101"].unit, "K")

    def testLoadCrex(self):
        t = wreport.Vartable.load_crex(os.path.join(os.environ["WREPORT_TABLES"], "B0000000000000024000.txt"))
        self.assertEqual(t["B12101"].unit, "C")

    def testGetBufr(self):
        t = wreport.Vartable.get_bufr(basename="B0000000000000024000")
        self.assertEqual(t["B12101"].unit, "K")

        t = wreport.Vartable.get_bufr(master_table_version_number=24)
        self.assertEqual(t["B12101"].unit, "K")

    def testGetCrex(self):
        t = wreport.Vartable.get_crex(basename="B0000000000000024000")
        self.assertEqual(t["B12101"].unit, "C")

        t = wreport.Vartable.get_crex(master_table_version_number_bufr=24)
        self.assertEqual(t["B12101"].unit, "C")

    def testCreate(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        self.assertEqual(os.path.basename(table.pathname), "B0000000000000024000.txt")
        self.assertEqual(os.path.basename(str(table)), "B0000000000000024000.txt")
        expected = "Vartable('{}')".format(table.pathname)
        self.assertEqual(repr(table), expected)

    def testContains(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        self.assertIn("B01001", table)
        self.assertNotIn("B63254", table)

#    def testIndexing(self):
#        table = wreport.Vartable("dballe")
#        info = table[0]
#        self.assertEqual(info.var, "B01001")

    def testLookup(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        info = table["B01001"]
        self.assertEqual(info.type, "integer")
        self.assertEqual(info.len, 3)
        self.assertEqual(info.unit, "NUMERIC")

    def testLookupMissing(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        with self.assertRaises(KeyError):
            table["B63254"]

#    def testIterate(self):
#        table = wreport.Vartable("dballe")
#        selected = None
#        count = 0
#        for entry in table:
#            if entry.var == "B12101":
#                selected = entry
#            count += 1
#        self.assertGreater(count, 100)
#        self.assertEqual(count, len(table))
#        self.assertIsNotNone(selected)



from setuptools import Extension, setup
import subprocess


def pkg_config_flags(options):
    return [
        s for s in subprocess.check_output(['pkg-config'] + options + ['libwreport']).decode().strip().split(" ") if s
    ]


wreport_module = Extension(
    '_wreport',
    sources=[
        "common.cc", "varinfo.cc", "vartable.cc", "var.cc", "wreport.cc"
    ],
    language="c++",
    extra_compile_args=pkg_config_flags(["--cflags"]) + ["-std=c++11"],
    extra_link_args=pkg_config_flags(["--libs"]),
)

setup(
    name="wreport",
    version="3.3.0",
    author="Enrico Zini",
    author_email="enrico@enricozini.org",
    maintainer="Emanuele Di Giacomo",
    maintainer_email="edigiacomo@arpa.emr.it",
    description="C++ library for working with weather reports",
    long_description="C++ library for working with weather reports",
    url="http://github.com/arpa-simc/wreport",
    license="GPLv2+",
    py_modules=[],
    packages=['wreport'],
    data_files=[],
    zip_safe=False,
    include_package_data=True,
    exclude_package_data={},
    ext_modules=[wreport_module],
)



#!/usr/bin/python3
import wreport
import unittest


class Varinfo(unittest.TestCase):
    def testEmptyVarinfo(self):
        self.assertRaises(NotImplementedError, wreport.Varinfo)

    def testData(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        info = table["B01001"]
        self.assertEqual(info.code, "B01001")
        self.assertEqual(info.desc, "WMO block number")
        self.assertEqual(info.unit, "NUMERIC")
        self.assertEqual(info.scale, 0)
        self.assertEqual(info.len, 3)
        self.assertEqual(info.bit_ref, 0)
        self.assertEqual(info.bit_len, 7)
        self.assertEqual(info.type, "integer")

    def testStringification(self):
        table = wreport.Vartable.get_bufr(master_table_version_number=24)
        info = table["B01001"]
        self.assertTrue(str(info).startswith("B01001"))
        self.assertTrue(repr(info).startswith("Varinfo('B01001"))



"""
wreport provides access to weather data in BUFR and CREX formats.

This is the Python interface to it
"""
from _wreport import (
        __version__,
        convert_units,
        Var,
        Varinfo,
        Vartable)

__all__ = ("convert_units", "Var", "Varinfo", "Vartable")



#!/usr/bin/python3
import wreport
import unittest


class Wreport(unittest.TestCase):
    def test_convert_units(self):
        self.assertAlmostEqual(wreport.convert_units("K", "C", 273.15), 0.0, places=4)
